@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -516,10 +518,10 @@ SpSubview<eT>::const_iterator::operator!=(const typename SpMat<eT>::const_row_it
 template<typename eT>
 inline
 arma_hot
-SpValProxy<SpSubview<eT> >
+SpValProxy< SpSubview<eT> >
 SpSubview<eT>::iterator::operator*()
   {
-  return SpValProxy<SpSubview<eT> >(
+  return SpValProxy< SpSubview<eT> >(
     const_iterator::row(),
     iterator_base::col(),
     access::rw(*iterator_base::M),
@@ -625,9 +627,9 @@ SpSubview<eT>::const_row_iterator::const_row_iterator(const SpSubview<eT>& in_M,
   // Since we don't know where the elements are in each row, we have to loop
   // across all columns looking for elements in row 0 and add to our sum, then
   // in row 1, and so forth, until we get to the desired position.
-  for (uword row = 0; row < iterator_base::M->n_rows; ++row)
+  for(uword row = 0; row < iterator_base::M->n_rows; ++row)
     {
-    for (uword col = 0; col < iterator_base::M->n_cols; ++col)
+    for(uword col = 0; col < iterator_base::M->n_cols; ++col)
       {
       // Find the first element with row greater than or equal to row + aux_row.
       const uword      col_offset = iterator_base::M->m.col_ptrs[col + aux_col    ];
@@ -636,24 +638,24 @@ SpSubview<eT>::const_row_iterator::const_row_iterator(const SpSubview<eT>& in_M,
       const uword* start_ptr = &iterator_base::M->m.row_indices[     col_offset];
       const uword*   end_ptr = &iterator_base::M->m.row_indices[next_col_offset];
 
-      if (start_ptr != end_ptr)
+      if(start_ptr != end_ptr)
         {
         const uword* pos_ptr = std::lower_bound(start_ptr, end_ptr, row + aux_row);
 
-        const uword offset = (pos_ptr - start_ptr);
+        const uword offset = uword(pos_ptr - start_ptr);
 
-        if (iterator_base::M->m.row_indices[col_offset + offset] == row + aux_row)
+        if(iterator_base::M->m.row_indices[col_offset + offset] == row + aux_row)
           {
           cur_actual_pos = col_offset + offset;
 
           // Increment position portably.
-          if (cur_pos == std::numeric_limits<uword>::max())
+          if(cur_pos == std::numeric_limits<uword>::max())
             cur_pos = 0;
           else
             ++cur_pos;
 
           // Do we terminate?
-          if (cur_pos == initial_pos)
+          if(cur_pos == initial_pos)
             {
             internal_row = row;
             iterator_base::internal_col = col;
@@ -702,7 +704,7 @@ SpSubview<eT>::const_row_iterator::const_row_iterator(const SpSubview<eT>& in_M,
   uword cur_min_col = 0;
   uword cur_actual_pos = 0;
 
-  for (uword col = 0; col < iterator_base::M->n_cols; ++col)
+  for(uword col = 0; col < iterator_base::M->n_cols; ++col)
     {
     // Find the first element with row greater than or equal to in_row.
     const uword      col_offset = iterator_base::M->m.col_ptrs[col + aux_col    ];
@@ -711,29 +713,29 @@ SpSubview<eT>::const_row_iterator::const_row_iterator(const SpSubview<eT>& in_M,
     const uword* start_ptr = &iterator_base::M->m.row_indices[     col_offset];
     const uword*   end_ptr = &iterator_base::M->m.row_indices[next_col_offset];
 
-    if (start_ptr != end_ptr)
+    if(start_ptr != end_ptr)
       {
       // First let us find the first element that is in the subview.
       const uword* first_subview_ptr = std::lower_bound(start_ptr, end_ptr, aux_row);
 
-      if (first_subview_ptr != end_ptr && (*first_subview_ptr) < aux_row + iterator_base::M->n_rows)
+      if(first_subview_ptr != end_ptr && (*first_subview_ptr) < aux_row + iterator_base::M->n_rows)
         {
         // There exists at least one element in the subview.
         const uword* pos_ptr = std::lower_bound(first_subview_ptr, end_ptr, aux_row + in_row);
 
         // This is the number of elements in the subview with row index less
         // than in_row.
-        cur_pos += (pos_ptr - first_subview_ptr);
+        cur_pos += uword(pos_ptr - first_subview_ptr);
 
-        if (pos_ptr != end_ptr && (*pos_ptr) < aux_row + iterator_base::M->n_rows)
+        if(pos_ptr != end_ptr && (*pos_ptr) < aux_row + iterator_base::M->n_rows)
           {
           // This is the row index of the first element in the column with row
           // index greater than or equal to in_row + aux_row.
-          if ((*pos_ptr) - aux_row < cur_min_row)
+          if((*pos_ptr) - aux_row < cur_min_row)
             {
             // If we are in the desired row but before the desired column, we
             // can't take this.
-            if (col >= in_col)
+            if(col >= in_col)
               {
               cur_min_row = (*pos_ptr) - aux_row;
               cur_min_col = col;
@@ -795,7 +797,7 @@ SpSubview<eT>::const_row_iterator::operator++()
   uword next_min_col = 0;
   uword next_actual_pos = 0;
 
-  for (uword col = iterator_base::internal_col + 1; col < M_n_cols; ++col)
+  for(uword col = iterator_base::internal_col + 1; col < M_n_cols; ++col)
     {
     // Find the first element with row greater than or equal to row.
     const uword      col_offset = iterator_base::M->m.col_ptrs[col + aux_col    ];
@@ -804,24 +806,24 @@ SpSubview<eT>::const_row_iterator::operator++()
     const uword* start_ptr = &iterator_base::M->m.row_indices[     col_offset];
     const uword* end_ptr   = &iterator_base::M->m.row_indices[next_col_offset];
 
-    if (start_ptr != end_ptr)
+    if(start_ptr != end_ptr)
       {
       // Find the first element in the column with row greater than or equal to
       // the current row.  Since this is a subview, it's possible that we may
       // find rows past the end of the subview.
       const uword* pos_ptr = std::lower_bound(start_ptr, end_ptr, internal_row + aux_row);
 
-      if (pos_ptr != end_ptr)
+      if(pos_ptr != end_ptr)
         {
         // We found something; is the row index correct?
-        if ((*pos_ptr) == internal_row + aux_row && (*pos_ptr) < aux_row + iterator_base::M->n_rows)
+        if((*pos_ptr) == internal_row + aux_row && (*pos_ptr) < aux_row + iterator_base::M->n_rows)
           {
           // Exact match---so we are done.
           iterator_base::internal_col = col;
           actual_pos = col_offset + (pos_ptr - start_ptr);
           return *this;
           }
-        else if ((*pos_ptr) < next_min_row + aux_row && (*pos_ptr) < aux_row + iterator_base::M->n_rows)
+        else if((*pos_ptr) < next_min_row + aux_row && (*pos_ptr) < aux_row + iterator_base::M->n_rows)
           {
           // The first element in this column is in a subsequent row, but it's
           // the minimum row we've seen so far.
@@ -829,7 +831,7 @@ SpSubview<eT>::const_row_iterator::operator++()
           next_min_col = col;
           next_actual_pos = col_offset + (pos_ptr - start_ptr);
           }
-        else if ((*pos_ptr) == next_min_row + aux_row && col < next_min_col && (*pos_ptr) < aux_row + iterator_base::M->n_rows)
+        else if((*pos_ptr) == next_min_row + aux_row && col < next_min_col && (*pos_ptr) < aux_row + iterator_base::M->n_rows)
           {
           // The first element in this column is in a subsequent row that we
           // already have another elemnt for, but the column index is less so
@@ -842,7 +844,7 @@ SpSubview<eT>::const_row_iterator::operator++()
     }
 
   // Restart the search in the next row.
-  for (uword col = 0; col <= iterator_base::internal_col; ++col)
+  for(uword col = 0; col <= iterator_base::internal_col; ++col)
     {
     // Find the first element with row greater than or equal to row + 1.
     const uword      col_offset = iterator_base::M->m.col_ptrs[col + aux_col    ];
@@ -851,14 +853,14 @@ SpSubview<eT>::const_row_iterator::operator++()
     const uword* start_ptr = &iterator_base::M->m.row_indices[     col_offset];
     const uword* end_ptr   = &iterator_base::M->m.row_indices[next_col_offset];
 
-    if (start_ptr != end_ptr)
+    if(start_ptr != end_ptr)
       {
       const uword* pos_ptr = std::lower_bound(start_ptr, end_ptr, internal_row + aux_row + 1);
 
-      if (pos_ptr != end_ptr)
+      if(pos_ptr != end_ptr)
         {
         // We found something in the column, but is the row index correct?
-        if ((*pos_ptr) == internal_row + aux_row + 1 && (*pos_ptr) < aux_row + iterator_base::M->n_rows)
+        if((*pos_ptr) == internal_row + aux_row + 1 && (*pos_ptr) < aux_row + iterator_base::M->n_rows)
           {
           // Exact match---so we are done.
           iterator_base::internal_col = col;
@@ -866,7 +868,7 @@ SpSubview<eT>::const_row_iterator::operator++()
           actual_pos = col_offset + (pos_ptr - start_ptr);
           return *this;
           }
-        else if ((*pos_ptr) < next_min_row + aux_row && (*pos_ptr) < aux_row + iterator_base::M->n_rows)
+        else if((*pos_ptr) < next_min_row + aux_row && (*pos_ptr) < aux_row + iterator_base::M->n_rows)
           {
           // The first element in this column is in a subsequent row, but it's
           // the minimum row we've seen so far.
@@ -874,7 +876,7 @@ SpSubview<eT>::const_row_iterator::operator++()
           next_min_col = col;
           next_actual_pos = col_offset + (pos_ptr - start_ptr);
           }
-        else if ((*pos_ptr) == next_min_row + aux_row && col < next_min_col && (*pos_ptr) < aux_row + iterator_base::M->n_rows)
+        else if((*pos_ptr) == next_min_row + aux_row && col < next_min_col && (*pos_ptr) < aux_row + iterator_base::M->n_rows)
           {
           // We've found a better column.
           next_min_col = col;
@@ -914,7 +916,7 @@ arma_hot
 typename SpSubview<eT>::const_row_iterator&
 SpSubview<eT>::const_row_iterator::operator--()
   {
-  if (iterator_base::internal_pos == 0)
+  if(iterator_base::internal_pos == 0)
     {
     // We are already at the beginning.
     return *this;
@@ -930,7 +932,7 @@ SpSubview<eT>::const_row_iterator::operator--()
   uword max_col = 0;
   uword next_actual_pos = 0;
 
-  for (uword col = iterator_base::internal_col; col >= 1; --col)
+  for(uword col = iterator_base::internal_col; col >= 1; --col)
     {
     // Find the first element with row greater than or equal to in_row + 1.
     const uword      col_offset = iterator_base::M->m.col_ptrs[col + aux_col - 1];
@@ -939,21 +941,21 @@ SpSubview<eT>::const_row_iterator::operator--()
     const uword* start_ptr = &iterator_base::M->m.row_indices[     col_offset];
     const uword* end_ptr   = &iterator_base::M->m.row_indices[next_col_offset];
 
-    if (start_ptr != end_ptr)
+    if(start_ptr != end_ptr)
       {
       // There are elements in this column.
       const uword* pos_ptr = std::lower_bound(start_ptr, end_ptr, internal_row + aux_row + 1);
 
-      if (pos_ptr != start_ptr)
+      if(pos_ptr != start_ptr)
         {
-        if (*(pos_ptr - 1) > max_row + aux_row)
+        if(*(pos_ptr - 1) > max_row + aux_row)
           {
           // There are elements in this column with row index < internal_row.
           max_row = *(pos_ptr - 1) - aux_row;
           max_col = col - 1;
           next_actual_pos = col_offset + (pos_ptr - 1 - start_ptr);
           }
-        else if (*(pos_ptr - 1) == max_row + aux_row && (col - 1) >= max_col)
+        else if(*(pos_ptr - 1) == max_row + aux_row && (col - 1) >= max_col)
           {
           max_col = col - 1;
           next_actual_pos = col_offset + (pos_ptr - 1 - start_ptr);
@@ -962,7 +964,7 @@ SpSubview<eT>::const_row_iterator::operator--()
       }
     }
 
-  for (uword col = iterator_base::M->n_cols - 1; col >= iterator_base::internal_col; --col)
+  for(uword col = iterator_base::M->n_cols - 1; col >= iterator_base::internal_col; --col)
     {
     // Find the first element with row greater than or equal to row + 1.
     const uword      col_offset = iterator_base::M->m.col_ptrs[col + aux_col    ];
@@ -971,21 +973,21 @@ SpSubview<eT>::const_row_iterator::operator--()
     const uword* start_ptr = &iterator_base::M->m.row_indices[     col_offset];
     const uword*   end_ptr = &iterator_base::M->m.row_indices[next_col_offset];
 
-    if (start_ptr != end_ptr)
+    if(start_ptr != end_ptr)
       {
       // There are elements in this column.
       const uword* pos_ptr = std::lower_bound(start_ptr, end_ptr, internal_row + aux_row);
 
-      if (pos_ptr != start_ptr)
+      if(pos_ptr != start_ptr)
         {
         // There are elements in this column with row index < internal_row.
-        if (*(pos_ptr - 1) > max_row + aux_row)
+        if(*(pos_ptr - 1) > max_row + aux_row)
           {
           max_row = *(pos_ptr - 1) - aux_row;
           max_col = col;
           next_actual_pos = col_offset + (pos_ptr - 1 - start_ptr);
           }
-        else if (*(pos_ptr - 1) == max_row + aux_row && col >= max_col)
+        else if(*(pos_ptr - 1) == max_row + aux_row && col >= max_col)
           {
           max_col = col;
           next_actual_pos = col_offset + (pos_ptr - 1 - start_ptr);
@@ -993,7 +995,7 @@ SpSubview<eT>::const_row_iterator::operator--()
         }
       }
 
-    if (col == 0) // Catch edge case that the loop termination condition won't.
+    if(col == 0) // Catch edge case that the loop termination condition won't.
       {
       break;
       }
@@ -1118,10 +1120,10 @@ SpSubview<eT>::const_row_iterator::operator!=(const typename SpMat<eT>::const_ro
 template<typename eT>
 inline
 arma_hot
-SpValProxy<SpSubview<eT> >
+SpValProxy< SpSubview<eT> >
 SpSubview<eT>::row_iterator::operator*()
   {
-  return SpValProxy<SpSubview<eT> >(
+  return SpValProxy< SpSubview<eT> >(
     const_row_iterator::internal_row,
     iterator_base::internal_col,
     access::rw(*iterator_base::M),
@@ -1182,4 +1184,5 @@ SpSubview<eT>::row_iterator::operator--(int)
   return tmp;
   }
 
+
 //! @}
