@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -36,6 +38,16 @@
 // #endif
 
 
+// NOTE: workaround for another thread_local issue on macOS
+// NOTE: where GCC (not Clang) may not have support for thread_local
+
+#if (defined(__APPLE__) && defined(__GNUG__) && !defined(__clang__))
+  #if !defined(ARMA_DONT_DISABLE_EXTERN_RNG)
+    #undef ARMA_USE_EXTERN_RNG
+  #endif
+#endif
+
+
 
 #if defined(ARMA_USE_EXTERN_RNG)
   extern thread_local std::mt19937_64 mt19937_64_instance;
@@ -489,7 +501,13 @@ struct arma_rng::randn
     {
     #if defined(ARMA_RNG_ALT)
       {
-      for(uword i=0; i < N; ++i)  { mem[i] = eT( arma_rng_alt::randn_val() ); }
+      // NOTE: old method to avoid regressions in user code that assumes specific sequence
+      
+      uword i, j;
+      
+      for(i=0, j=1; j < N; i+=2, j+=2)  { arma_rng_alt::randn_dual_val( mem[i], mem[j] ); }
+      
+      if(i < N)  { mem[i] = eT( arma_rng_alt::randn_val() ); }
       }
     #elif defined(ARMA_USE_EXTERN_RNG)
       {
@@ -753,7 +771,7 @@ struct arma_rng::randg
       std::mt19937_64                 local_engine;
       std::gamma_distribution<double> local_g_distr(a,b);
       
-      local_engine.seed( local_seed_type(std::rand()) );
+      local_engine.seed( local_seed_type(arma_rng::randi<local_seed_type>()) );
       
       for(uword i=0; i<N; ++i)  { mem[i] = eT(local_g_distr(local_engine)); }
       }
