@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -135,7 +137,7 @@ static
 void
 arma_stop_logic_error(const T1& x)
   {
-  #if defined(ARMA_PRINT_ERRORS)
+  #if (defined(ARMA_PRINT_EXCEPTIONS) && defined(ARMA_PRINT_ERRORS))
     {
     get_cerr_stream() << "\nerror: " << x << std::endl;
     }
@@ -146,6 +148,17 @@ arma_stop_logic_error(const T1& x)
 
 
 
+arma_cold
+arma_noinline
+static
+void
+arma_stop_logic_error(const char* x, const char* y)
+  {
+  arma_stop_logic_error( std::string(x) + std::string(y) );
+  }
+
+
+
 //! print a message to get_cerr_stream() and throw logic_error exception
 template<typename T1>
 arma_cold
@@ -154,7 +167,7 @@ static
 void
 arma_stop_bounds_error(const T1& x)
   {
-  #if defined(ARMA_PRINT_ERRORS)
+  #if (defined(ARMA_PRINT_EXCEPTIONS) && defined(ARMA_PRINT_ERRORS))
     {
     get_cerr_stream() << "\nerror: " << x << std::endl;
     }
@@ -173,7 +186,7 @@ static
 void
 arma_stop_bad_alloc(const T1& x)
   {
-  #if defined(ARMA_PRINT_ERRORS)
+  #if (defined(ARMA_PRINT_EXCEPTIONS) && defined(ARMA_PRINT_ERRORS))
     {
     get_cerr_stream() << "\nerror: " << x << std::endl;
     }
@@ -196,7 +209,7 @@ static
 void
 arma_stop_runtime_error(const T1& x)
   {
-  #if defined(ARMA_PRINT_ERRORS)
+  #if (defined(ARMA_PRINT_EXCEPTIONS) && defined(ARMA_PRINT_ERRORS))
     {
     get_cerr_stream() << "\nerror: " << x << std::endl;
     }
@@ -410,6 +423,55 @@ arma_warn(const T1& arg1, const T2& arg2, const T3& arg3, const T4& arg4)
 
 
 
+//
+// arma_warn_level
+
+
+template<typename T1>
+inline
+void
+arma_warn_level(const uword level, const T1& arg1)
+  {
+  constexpr uword config_level = (sword(ARMA_WARN_LEVEL) > 0) ? uword(ARMA_WARN_LEVEL) : uword(0);
+  
+  if((config_level > 0) && (level <= config_level))  { arma_warn(arg1); }
+  }
+
+
+template<typename T1, typename T2>
+inline
+void
+arma_warn_level(const uword level, const T1& arg1, const T2& arg2)
+  {
+  constexpr uword config_level = (sword(ARMA_WARN_LEVEL) > 0) ? uword(ARMA_WARN_LEVEL) : uword(0);
+  
+  if((config_level > 0) && (level <= config_level))  { arma_warn(arg1,arg2); }
+  }
+
+
+template<typename T1, typename T2, typename T3>
+inline
+void
+arma_warn_level(const uword level, const T1& arg1, const T2& arg2, const T3& arg3)
+  {
+  constexpr uword config_level = (sword(ARMA_WARN_LEVEL) > 0) ? uword(ARMA_WARN_LEVEL) : uword(0);
+  
+  if((config_level > 0) && (level <= config_level))  { arma_warn(arg1,arg2,arg3); }
+  }
+
+
+template<typename T1, typename T2, typename T3, typename T4>
+inline
+void
+arma_warn_level(const uword level, const T1& arg1, const T2& arg2, const T3& arg3, const T4& arg4)
+  {
+  constexpr uword config_level = (sword(ARMA_WARN_LEVEL) > 0) ? uword(ARMA_WARN_LEVEL) : uword(0);
+  
+  if((config_level > 0) && (level <= config_level))  { arma_warn(arg1,arg2,arg3,arg4); }
+  }
+
+
+
 //
 // arma_check
 
@@ -424,13 +486,12 @@ arma_check(const bool state, const T1& x)
   }
 
 
-template<typename T1, typename T2>
 arma_hot
 inline
 void
-arma_check(const bool state, const T1& x, const T2& y)
+arma_check(const bool state, const char* x, const char* y)
   {
-  if(state)  { arma_stop_logic_error( std::string(x) + std::string(y) ); }
+  if(state)  { arma_stop_logic_error(x,y); }
   }
 
 
@@ -877,7 +938,7 @@ arma_assert_same_size(const ProxyCube<eT1>& A, const ProxyCube<eT2>& B, const ch
 
 
 //
-// functions for checking whether a cube or subcube can be interpreted as a matrix (i.e. single slice)
+// functions for checking whether a cube or subcube can be interpreted as a matrix (ie. single slice)
 
 
 
@@ -1309,10 +1370,9 @@ arma_assert_atlas_size(const T1& A, const T2& B)
 
 #if defined(ARMA_NO_DEBUG)
   
-  #undef ARMA_EXTRA_DEBUG
-  
   #define arma_debug_print                   true ? (void)0 : arma_print
   #define arma_debug_warn                    true ? (void)0 : arma_warn
+  #define arma_debug_warn_level              true ? (void)0 : arma_warn_level
   #define arma_debug_check                   true ? (void)0 : arma_check
   #define arma_debug_check_bounds            true ? (void)0 : arma_check_bounds
   #define arma_debug_set_error               true ? (void)0 : arma_set_error
@@ -1327,6 +1387,7 @@ arma_assert_atlas_size(const T1& A, const T2& B)
   
   #define arma_debug_print                 arma_print
   #define arma_debug_warn                  arma_warn
+  #define arma_debug_warn_level            arma_warn_level
   #define arma_debug_check                 arma_check
   #define arma_debug_check_bounds          arma_check_bounds
   #define arma_debug_set_error             arma_set_error
@@ -1343,19 +1404,18 @@ arma_assert_atlas_size(const T1& A, const T2& B)
 
 #if defined(ARMA_EXTRA_DEBUG)
   
+  #undef  ARMA_WARN_LEVEL
+  #define ARMA_WARN_LEVEL 3
+  
   #define arma_extra_debug_sigprint       arma_sigprint(ARMA_FNSIG); arma_bktprint
   #define arma_extra_debug_sigprint_this  arma_sigprint(ARMA_FNSIG); arma_thisprint
   #define arma_extra_debug_print          arma_print
-  #define arma_extra_debug_warn           arma_warn
-  #define arma_extra_debug_check          arma_check
 
 #else
   
   #define arma_extra_debug_sigprint        true ? (void)0 : arma_bktprint
   #define arma_extra_debug_sigprint_this   true ? (void)0 : arma_thisprint
   #define arma_extra_debug_print           true ? (void)0 : arma_print
-  #define arma_extra_debug_warn            true ? (void)0 : arma_warn
-  #define arma_extra_debug_check           true ? (void)0 : arma_check
  
 #endif
 
