@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -19,60 +21,18 @@
 
 
 
-template<typename T1>
-arma_warn_unused
-arma_inline
-const mtOp<uword, T1, op_all>
-all
-  (
-  const T1&   X,
-  const uword dim = 0,
-  const typename enable_if< is_arma_type<T1>::value       == true  >::result* junk1 = 0,
-  const typename enable_if< resolves_to_vector<T1>::value == false >::result* junk2 = 0
-  )
-  {
-  arma_extra_debug_sigprint();
-  arma_ignore(junk1);
-  arma_ignore(junk2);
-  
-  return mtOp<uword, T1, op_all>(X, dim, 0);
-  }
-
-
-
-template<typename T1>
-arma_warn_unused
-arma_inline
-const mtOp<uword, T1, op_all>
-all
-  (
-  const T1&   X,
-  const uword dim,
-  const typename enable_if<resolves_to_vector<T1>::value == true>::result* junk = 0
-  )
-  {
-  arma_extra_debug_sigprint();
-  arma_ignore(junk);
-  
-  return mtOp<uword, T1, op_all>(X, dim, 0);
-  }
-
-
-
 template<typename T1>
 arma_warn_unused
 inline
-bool
-all
-  (
-  const T1& X,
-  const arma_empty_class junk1 = arma_empty_class(),
-  const typename enable_if<resolves_to_vector<T1>::value == true>::result* junk2 = 0
-  )
+typename
+enable_if2
+  <
+  is_arma_type<T1>::value && resolves_to_vector<T1>::yes,
+  bool
+  >::result
+all(const T1& X)
   {
   arma_extra_debug_sigprint();
-  arma_ignore(junk1);
-  arma_ignore(junk2);
   
   return op_all::all_vec(X);
   }
@@ -81,14 +41,18 @@ all
 
 template<typename T1>
 arma_warn_unused
-inline
-bool
-all(const mtOp<uword, T1, op_all>& in)
+arma_inline
+typename
+enable_if2
+  <
+  is_arma_type<T1>::value && resolves_to_vector<T1>::no,
+  const mtOp<uword, T1, op_all>
+  >::result
+all(const T1& X)
   {
   arma_extra_debug_sigprint();
-  arma_extra_debug_print("all(): two consecutive calls to all() detected");
   
-  return op_all::all_vec(in.m);
+  return mtOp<uword, T1, op_all>(X, 0, 0);
   }
 
 
@@ -96,12 +60,34 @@ all(const mtOp<uword, T1, op_all>& in)
 template<typename T1>
 arma_warn_unused
 arma_inline
-const Op< mtOp<uword, T1, op_all>, op_all>
-all(const mtOp<uword, T1, op_all>& in, const uword dim)
+typename
+enable_if2
+  <
+  is_arma_type<T1>::value,
+  const mtOp<uword, T1, op_all>
+  >::result
+all(const T1& X, const uword dim)
   {
   arma_extra_debug_sigprint();
   
-  return mtOp<uword, mtOp<uword, T1, op_all>, op_all>(in, dim, 0);
+  return mtOp<uword, T1, op_all>(X, dim, 0);
+  }
+
+
+
+//! for compatibility purposes: allows compiling user code designed for earlier versions of Armadillo
+template<typename T>
+arma_warn_unused
+arma_inline
+typename
+enable_if2
+  <
+  is_supported_elem_type<T>::value,
+  bool
+  >::result
+all(const T& val)
+  {
+  return (val != T(0));
   }
 
 
