@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -25,14 +27,32 @@ arma_inline
 typename
 enable_if2
   <
-  is_arma_type<T1>::value,
-  const Op<T1, op_cumsum_default>
+  is_arma_type<T1>::value && resolves_to_vector<T1>::yes,
+  const Op<T1, op_cumsum_vec>
   >::result
 cumsum(const T1& X)
   {
   arma_extra_debug_sigprint();
   
-  return Op<T1, op_cumsum_default>(X);
+  return Op<T1, op_cumsum_vec>(X);
+  }
+
+
+
+template<typename T1>
+arma_warn_unused
+arma_inline
+typename
+enable_if2
+  <
+  is_arma_type<T1>::value && resolves_to_vector<T1>::no,
+  const Op<T1, op_cumsum>
+  >::result
+cumsum(const T1& X)
+  {
+  arma_extra_debug_sigprint();
+  
+  return Op<T1, op_cumsum>(X, 0, 0);
   }
 
 
@@ -58,7 +78,7 @@ cumsum(const T1& X, const uword dim)
 template<typename T>
 arma_warn_unused
 arma_inline
-const typename arma_scalar_only<T>::result &
+typename arma_scalar_only<T>::result
 cumsum(const T& x)
   {
   return x;
