@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -34,16 +36,12 @@ nonzeros(const Base<typename T1::elem_type,T1>& X)
 template<typename T1>
 arma_warn_unused
 inline
-Col<typename T1::elem_type>
+const SpToDOp<T1, op_nonzeros_spmat>
 nonzeros(const SpBase<typename T1::elem_type,T1>& X)
   {
   arma_extra_debug_sigprint();
   
-  Col<typename T1::elem_type> out;
-  
-  op_nonzeros::apply_noalias(out, X.get_ref());
-  
-  return out;
+  return SpToDOp<T1, op_nonzeros_spmat>(X.get_ref());
   }
 
 
