@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -26,9 +28,9 @@ class spglue_times
   template<typename T1, typename T2>
   struct traits
     {
-    static const bool is_row  = T1::is_row;
-    static const bool is_col  = T2::is_col;
-    static const bool is_xvec = false;
+    static constexpr bool is_row  = T1::is_row;
+    static constexpr bool is_col  = T2::is_col;
+    static constexpr bool is_xvec = false;
     };
   
   template<typename T1, typename T2>
@@ -50,9 +52,9 @@ class spglue_times_misc
   template<typename T1, typename T2>
   struct traits
     {
-    static const bool is_row  = T1::is_row;
-    static const bool is_col  = T2::is_col;
-    static const bool is_xvec = false;
+    static constexpr bool is_row  = T1::is_row;
+    static constexpr bool is_col  = T2::is_col;
+    static constexpr bool is_xvec = false;
     };
   
   template<typename T1, typename T2>
@@ -71,9 +73,9 @@ class spglue_times_mixed
   template<typename T1, typename T2>
   struct traits
     {
-    static const bool is_row  = T1::is_row;
-    static const bool is_col  = T2::is_col;
-    static const bool is_xvec = false;
+    static constexpr bool is_row  = T1::is_row;
+    static constexpr bool is_col  = T2::is_col;
+    static constexpr bool is_xvec = false;
     };
   
   template<typename T1, typename T2>
