@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
 // Copyright 2008-2016 National ICT Australia (NICTA)
 // 
@@ -19,7 +21,7 @@
 
 
 //! Class for storing data required to construct or apply operations to a submatrix
-//! (i.e. where the submatrix starts and ends as well as a reference/pointer to the original matrix),
+//! (ie. where the submatrix starts and ends as well as a reference/pointer to the original matrix),
 template<typename eT>
 class subview : public Base< eT, subview<eT> >
   {
@@ -51,6 +53,9 @@ class subview : public Base< eT, subview<eT> >
   inline ~subview();
   inline  subview() = delete;
   
+  inline  subview(const subview&  in);
+  inline  subview(      subview&& in);
+  
   template<typename op_type             > inline void inplace_op(const eT           val                        );
   template<typename op_type, typename T1> inline void inplace_op(const Base<eT,T1>& x,   const char* identifier);
   template<typename op_type             > inline void inplace_op(const subview<eT>& x,   const char* identifier);
@@ -84,6 +89,9 @@ class subview : public Base< eT, subview<eT> >
   template<typename T1, typename gen_type>
   inline typename enable_if2< is_same_type<typename T1::elem_type, eT>::value, void>::result operator=(const Gen<T1,gen_type>& x);
   
+  inline void operator=(const std::initializer_list<eT>& list);
+  inline void operator=(const std::initializer_list< std::initializer_list<eT> >& list);
+  
   
   inline static void extract(Mat<eT>& out, const subview& in);
   
@@ -102,6 +110,8 @@ class subview : public Base< eT, subview<eT> >
   
   inline void clean(const pod_type threshold);
   
+  inline void clamp(const eT min_val, const eT max_val);
+  
   inline void fill(const eT val);
   inline void zeros();
   inline void ones();
@@ -109,25 +119,25 @@ class subview : public Base< eT, subview<eT> >
   inline void randu();
   inline void randn();
   
-  inline eT  at_alt    (const uword ii) const;
+  inline arma_warn_unused eT  at_alt    (const uword ii) const;
   
-  inline eT& operator[](const uword ii);
-  inline eT  operator[](const uword ii) const;
+  inline arma_warn_unused eT& operator[](const uword ii);
+  inline arma_warn_unused eT  operator[](const uword ii) const;
   
-  inline eT& operator()(const uword ii);
-  inline eT  operator()(const uword ii) const;
+  inline arma_warn_unused eT& operator()(const uword ii);
+  inline arma_warn_unused eT  operator()(const uword ii) const;
   
-  inline eT& operator()(const uword in_row, const uword in_col);
-  inline eT  operator()(const uword in_row, const uword in_col) const;
+  inline arma_warn_unused eT& operator()(const uword in_row, const uword in_col);
+  inline arma_warn_unused eT  operator()(const uword in_row, const uword in_col) const;
   
-  inline eT&         at(const uword in_row, const uword in_col);
-  inline eT          at(const uword in_row, const uword in_col) const;
+  inline arma_warn_unused eT&         at(const uword in_row, const uword in_col);
+  inline arma_warn_unused eT          at(const uword in_row, const uword in_col) const;
   
-  inline eT&         front();
-  inline eT          front() const;
+  inline arma_warn_unused eT& front();
+  inline arma_warn_unused eT  front() const;
   
-  inline eT&         back();
-  inline eT          back() const;
+  inline arma_warn_unused eT& back();
+  inline arma_warn_unused eT  back() const;
   
   arma_inline       eT* colptr(const uword in_col);
   arma_inline const eT* colptr(const uword in_col) const;
@@ -364,18 +374,19 @@ class subview_col : public subview<eT>
   inline void operator= (const subview<eT>& x);
   inline void operator= (const subview_col& x);
   inline void operator= (const eT val);
+  inline void operator= (const std::initializer_list<eT>& list);
   
-  template<typename T1>
-  inline void operator= (const Base<eT,T1>& x);
+  template<typename T1> inline void operator= (const   Base<eT,T1>& x);
+  template<typename T1> inline void operator= (const SpBase<eT,T1>& x);
   
   template<typename T1, typename gen_type>
   inline typename enable_if2< is_same_type<typename T1::elem_type, eT>::value, void>::result operator=(const Gen<T1,gen_type>& x);
   
-  arma_inline const Op<subview_col<eT>,op_htrans>  t() const;
-  arma_inline const Op<subview_col<eT>,op_htrans> ht() const;
-  arma_inline const Op<subview_col<eT>,op_strans> st() const;
+  arma_inline arma_warn_unused const Op<subview_col<eT>,op_htrans>  t() const;
+  arma_inline arma_warn_unused const Op<subview_col<eT>,op_htrans> ht() const;
+  arma_inline arma_warn_unused const Op<subview_col<eT>,op_strans> st() const;
   
-  arma_inline const Op<subview_col<eT>,op_strans> as_row() const;
+  arma_inline arma_warn_unused const Op<subview_col<eT>,op_strans> as_row() const;
   
   inline void fill(const eT val);
   inline void zeros();
@@ -422,6 +433,9 @@ class subview_col : public subview<eT>
   inline arma_warn_unused uword index_min() const;
   inline arma_warn_unused uword index_max() const;
   
+  inline  subview_col(const subview_col&  in);
+  inline  subview_col(      subview_col&& in);
+  
   
   protected:
   
@@ -437,6 +451,67 @@ class subview_col : public subview<eT>
 
 
 
+template<typename eT>
+class subview_cols : public subview<eT>
+  {
+  public:
+  
+  typedef eT                                       elem_type;
+  typedef typename get_pod_type<elem_type>::result pod_type;
+  
+  static constexpr bool is_row  = false;
+  static constexpr bool is_col  = false;
+  static constexpr bool is_xvec = false;
+  
+  inline  subview_cols(const subview_cols&  in);
+  inline  subview_cols(      subview_cols&& in);
+  
+  inline void operator= (const subview<eT>&  x);
+  inline void operator= (const subview_cols& x);
+  inline void operator= (const eT val);
+  inline void operator= (const std::initializer_list<eT>& list);
+  inline void operator= (const std::initializer_list< std::initializer_list<eT> >& list);
+  
+  template<typename T1> inline void operator= (const   Base<eT,T1>& x);
+  template<typename T1> inline void operator= (const SpBase<eT,T1>& x);
+  
+  template<typename T1, typename gen_type>
+  inline typename enable_if2< is_same_type<typename T1::elem_type, eT>::value, void>::result operator=(const Gen<T1,gen_type>& x);
+  
+  arma_inline arma_warn_unused const Op<subview_cols<eT>,op_htrans>  t() const;
+  arma_inline arma_warn_unused const Op<subview_cols<eT>,op_htrans> ht() const;
+  arma_inline arma_warn_unused const Op<subview_cols<eT>,op_strans> st() const;
+  
+  arma_inline arma_warn_unused const Op<subview_cols<eT>,op_vectorise_col> as_col() const;
+  
+  inline arma_warn_unused eT  at_alt    (const uword ii) const;
+  
+  inline arma_warn_unused eT& operator[](const uword ii);
+  inline arma_warn_unused eT  operator[](const uword ii) const;
+  
+  inline arma_warn_unused eT& operator()(const uword ii);
+  inline arma_warn_unused eT  operator()(const uword ii) const;
+  
+  inline arma_warn_unused eT& operator()(const uword in_row, const uword in_col);
+  inline arma_warn_unused eT  operator()(const uword in_row, const uword in_col) const;
+  
+  inline arma_warn_unused eT&         at(const uword in_row, const uword in_col);
+  inline arma_warn_unused eT          at(const uword in_row, const uword in_col) const;
+  
+  arma_inline       eT* colptr(const uword in_col);
+  arma_inline const eT* colptr(const uword in_col) const;
+  
+  protected:
+  
+  inline subview_cols(const Mat<eT>& in_m, const uword in_col1, const uword in_n_cols);
+  inline subview_cols() = delete;
+  
+  friend class Mat<eT>;
+  friend class subview<eT>;
+  };
+
+
+
 template<typename eT>
 class subview_row : public subview<eT>
   {
@@ -452,18 +527,19 @@ class subview_row : public subview<eT>
   inline void operator= (const subview<eT>& x);
   inline void operator= (const subview_row& x);
   inline void operator= (const eT val);
+  inline void operator= (const std::initializer_list<eT>& list);
   
-  template<typename T1>
-  inline void operator= (const Base<eT,T1>& x);
+  template<typename T1> inline void operator= (const   Base<eT,T1>& x);
+  template<typename T1> inline void operator= (const SpBase<eT,T1>& x);
   
   template<typename T1, typename gen_type>
   inline typename enable_if2< is_same_type<typename T1::elem_type, eT>::value, void>::result operator=(const Gen<T1,gen_type>& x);
   
-  arma_inline const Op<subview_row<eT>,op_htrans>  t() const;
-  arma_inline const Op<subview_row<eT>,op_htrans> ht() const;
-  arma_inline const Op<subview_row<eT>,op_strans> st() const;
+  arma_inline arma_warn_unused const Op<subview_row<eT>,op_htrans>  t() const;
+  arma_inline arma_warn_unused const Op<subview_row<eT>,op_htrans> ht() const;
+  arma_inline arma_warn_unused const Op<subview_row<eT>,op_strans> st() const;
   
-  arma_inline const Op<subview_row<eT>,op_strans> as_col() const;
+  arma_inline arma_warn_unused const Op<subview_row<eT>,op_strans> as_col() const;
   
   inline eT  at_alt    (const uword i) const;
   
@@ -505,6 +581,9 @@ class subview_row : public subview<eT>
   inline typename subview<eT>::const_row_iterator  end() const;
   inline typename subview<eT>::const_row_iterator cend() const;
   
+  inline  subview_row(const subview_row&  in);
+  inline  subview_row(      subview_row&& in);
+  
   
   protected:
   
@@ -534,8 +613,9 @@ class subview_row_strans : public Base< eT, subview_row_strans<eT> >
   
   arma_aligned const subview_row<eT>& sv_row;
   
-         const     uword n_rows;     // equal to n_elem
-         const     uword n_elem;
+  const uword n_rows;     // equal to n_elem
+  const uword n_elem;
+  
   static constexpr uword n_cols = 1;
   
   
@@ -568,8 +648,9 @@ class subview_row_htrans : public Base< eT, subview_row_htrans<eT> >
   
   arma_aligned const subview_row<eT>& sv_row;
   
-         const     uword n_rows;     // equal to n_elem
-         const     uword n_elem;
+  const uword n_rows;     // equal to n_elem
+  const uword n_elem;
+  
   static constexpr uword n_cols = 1;
   
   
