@@ -1,11 +1,19 @@
-// Copyright (C) 2010-2016 National ICT Australia (NICTA)
+// SPDX-License-Identifier: Apache-2.0
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// -------------------------------------------------------------------
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// Written by Conrad Sanderson - http://conradsanderson.id.au
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
+// 
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 //! \addtogroup subview_elem1
@@ -27,8 +35,6 @@ subview_elem1<eT,T1>::subview_elem1(const Mat<eT>& in_m, const Base<uword,T1>& i
   , a(in_a)
   {
   arma_extra_debug_sigprint();
-  
-  // TODO: refactor to unwrap 'in_a' instead of storing a ref to it; this will allow removal of carrying T1 around and repetition of size checks
   }
 
 
@@ -64,7 +70,7 @@ subview_elem1<eT,T1>::inplace_op(const eT val)
   arma_debug_check
     (
     ( (aa.is_vec() == false) && (aa.is_empty() == false) ),
-    "Mat::elem(): given object is not a vector"
+    "Mat::elem(): given object must be a vector"
     );
   
   const uword* aa_mem    = aa.memptr();
@@ -76,7 +82,7 @@ subview_elem1<eT,T1>::inplace_op(const eT val)
     const uword ii = aa_mem[iq];
     const uword jj = aa_mem[jq];
     
-    arma_debug_check( ( (ii >= m_n_elem) || (jj >= m_n_elem) ), "Mat::elem(): index out of bounds" );
+    arma_debug_check_bounds( ( (ii >= m_n_elem) || (jj >= m_n_elem) ), "Mat::elem(): index out of bounds" );
     
     if(is_same_type<op_type, op_internal_equ  >::yes) { m_mem[ii] =  val; m_mem[jj] =  val; }
     if(is_same_type<op_type, op_internal_plus >::yes) { m_mem[ii] += val; m_mem[jj] += val; }
@@ -89,7 +95,7 @@ subview_elem1<eT,T1>::inplace_op(const eT val)
     {
     const uword ii = aa_mem[iq];
     
-    arma_debug_check( (ii >= m_n_elem) , "Mat::elem(): index out of bounds" ); 
+    arma_debug_check_bounds( (ii >= m_n_elem) , "Mat::elem(): index out of bounds" ); 
     
     if(is_same_type<op_type, op_internal_equ  >::yes) { m_mem[ii] =  val; }
     if(is_same_type<op_type, op_internal_plus >::yes) { m_mem[ii] += val; }
@@ -137,7 +143,7 @@ subview_elem1<eT,T1>::inplace_op(const subview_elem1<eT,T2>& x)
     arma_debug_check
       (
       ( ((s_aa.is_vec() == false) && (s_aa.is_empty() == false)) || ((x_aa.is_vec() == false) && (x_aa.is_empty() == false)) ),
-      "Mat::elem(): given object is not a vector"
+      "Mat::elem(): given object must be a vector"
       );
     
     const uword* s_aa_mem = s_aa.memptr();
@@ -163,7 +169,7 @@ subview_elem1<eT,T1>::inplace_op(const subview_elem1<eT,T2>& x)
       const uword x_ii = x_aa_mem[iq];
       const uword x_jj = x_aa_mem[jq];
       
-      arma_debug_check
+      arma_debug_check_bounds
         (
         (s_ii >= s_m_n_elem) || (s_jj >= s_m_n_elem) || (x_ii >= x_m_n_elem) || (x_jj >= x_m_n_elem),
         "Mat::elem(): index out of bounds"
@@ -181,7 +187,7 @@ subview_elem1<eT,T1>::inplace_op(const subview_elem1<eT,T2>& x)
       const uword s_ii = s_aa_mem[iq];
       const uword x_ii = x_aa_mem[iq];
       
-      arma_debug_check
+      arma_debug_check_bounds
         (
         ( (s_ii >= s_m_n_elem) || (x_ii >= x_m_n_elem) ),
         "Mat::elem(): index out of bounds"
@@ -217,7 +223,7 @@ subview_elem1<eT,T1>::inplace_op(const Base<eT,T2>& x)
   arma_debug_check
     (
     ( (aa.is_vec() == false) && (aa.is_empty() == false) ),
-    "Mat::elem(): given object is not a vector"
+    "Mat::elem(): given object must be a vector"
     );
   
   const uword* aa_mem    = aa.memptr();
@@ -239,7 +245,7 @@ subview_elem1<eT,T1>::inplace_op(const Base<eT,T2>& x)
       const uword ii = aa_mem[iq];
       const uword jj = aa_mem[jq];
       
-      arma_debug_check( ( (ii >= m_n_elem) || (jj >= m_n_elem) ), "Mat::elem(): index out of bounds" );
+      arma_debug_check_bounds( ( (ii >= m_n_elem) || (jj >= m_n_elem) ), "Mat::elem(): index out of bounds" );
       
       if(is_same_type<op_type, op_internal_equ  >::yes) { m_mem[ii] =  X[iq]; m_mem[jj]  = X[jq]; }
       if(is_same_type<op_type, op_internal_plus >::yes) { m_mem[ii] += X[iq]; m_mem[jj] += X[jq]; }
@@ -252,7 +258,7 @@ subview_elem1<eT,T1>::inplace_op(const Base<eT,T2>& x)
       {
       const uword ii = aa_mem[iq];
       
-      arma_debug_check( (ii >= m_n_elem) , "Mat::elem(): index out of bounds" );
+      arma_debug_check_bounds( (ii >= m_n_elem) , "Mat::elem(): index out of bounds" );
       
       if(is_same_type<op_type, op_internal_equ  >::yes) { m_mem[ii] =  X[iq]; }
       if(is_same_type<op_type, op_internal_plus >::yes) { m_mem[ii] += X[iq]; }
@@ -276,7 +282,7 @@ subview_elem1<eT,T1>::inplace_op(const Base<eT,T2>& x)
       const uword ii = aa_mem[iq];
       const uword jj = aa_mem[jq];
       
-      arma_debug_check( ( (ii >= m_n_elem) || (jj >= m_n_elem) ), "Mat::elem(): index out of bounds" );
+      arma_debug_check_bounds( ( (ii >= m_n_elem) || (jj >= m_n_elem) ), "Mat::elem(): index out of bounds" );
       
       if(is_same_type<op_type, op_internal_equ  >::yes) { m_mem[ii] =  X[iq]; m_mem[jj]  = X[jq]; }
       if(is_same_type<op_type, op_internal_plus >::yes) { m_mem[ii] += X[iq]; m_mem[jj] += X[jq]; }
@@ -289,7 +295,7 @@ subview_elem1<eT,T1>::inplace_op(const Base<eT,T2>& x)
       {
       const uword ii = aa_mem[iq];
       
-      arma_debug_check( (ii >= m_n_elem) , "Mat::elem(): index out of bounds" );
+      arma_debug_check_bounds( (ii >= m_n_elem) , "Mat::elem(): index out of bounds" );
       
       if(is_same_type<op_type, op_internal_equ  >::yes) { m_mem[ii] =  X[iq]; }
       if(is_same_type<op_type, op_internal_plus >::yes) { m_mem[ii] += X[iq]; }
@@ -355,7 +361,7 @@ subview_elem1<eT,T1>::replace(const eT old_val, const eT new_val)
   arma_debug_check
     (
     ( (aa.is_vec() == false) && (aa.is_empty() == false) ),
-    "Mat::elem(): given object is not a vector"
+    "Mat::elem(): given object must be a vector"
     );
   
   const uword* aa_mem    = aa.memptr();
@@ -367,7 +373,7 @@ subview_elem1<eT,T1>::replace(const eT old_val, const eT new_val)
       {
       const uword ii = aa_mem[iq];
       
-      arma_debug_check( (ii >= m_n_elem), "Mat::elem(): index out of bounds" );
+      arma_debug_check_bounds( (ii >= m_n_elem), "Mat::elem(): index out of bounds" );
       
       eT& val = m_mem[ii];
       
@@ -380,7 +386,7 @@ subview_elem1<eT,T1>::replace(const eT old_val, const eT new_val)
       {
       const uword ii = aa_mem[iq];
       
-      arma_debug_check( (ii >= m_n_elem), "Mat::elem(): index out of bounds" );
+      arma_debug_check_bounds( (ii >= m_n_elem), "Mat::elem(): index out of bounds" );
       
       eT& val = m_mem[ii];
       
@@ -391,6 +397,38 @@ subview_elem1<eT,T1>::replace(const eT old_val, const eT new_val)
 
 
 
+template<typename eT, typename T1>
+inline
+void
+subview_elem1<eT,T1>::clean(const pod_type threshold)
+  {
+  arma_extra_debug_sigprint();
+  
+  Mat<eT> tmp(*this);
+  
+  tmp.clean(threshold);
+  
+  (*this).operator=(tmp);
+  }
+
+
+
+template<typename eT, typename T1>
+inline
+void
+subview_elem1<eT,T1>::clamp(const eT min_val, const eT max_val)
+  {
+  arma_extra_debug_sigprint();
+  
+  Mat<eT> tmp(*this);
+  
+  tmp.clamp(min_val, max_val);
+  
+  (*this).operator=(tmp);
+  }
+
+
+
 template<typename eT, typename T1>
 inline
 void
@@ -445,7 +483,7 @@ subview_elem1<eT,T1>::randu()
   arma_debug_check
     (
     ( (aa.is_vec() == false) && (aa.is_empty() == false) ),
-    "Mat::elem(): given object is not a vector"
+    "Mat::elem(): given object must be a vector"
     );
   
   const uword* aa_mem    = aa.memptr();
@@ -457,7 +495,7 @@ subview_elem1<eT,T1>::randu()
     const uword ii = aa_mem[iq];
     const uword jj = aa_mem[jq];
     
-    arma_debug_check( ( (ii >= m_n_elem) || (jj >= m_n_elem) ), "Mat::elem(): index out of bounds" );
+    arma_debug_check_bounds( ( (ii >= m_n_elem) || (jj >= m_n_elem) ), "Mat::elem(): index out of bounds" );
     
     const eT val1 = eT(arma_rng::randu<eT>());
     const eT val2 = eT(arma_rng::randu<eT>());
@@ -470,7 +508,7 @@ subview_elem1<eT,T1>::randu()
     {
     const uword ii = aa_mem[iq];
     
-    arma_debug_check( (ii >= m_n_elem) , "Mat::elem(): index out of bounds" ); 
+    arma_debug_check_bounds( (ii >= m_n_elem) , "Mat::elem(): index out of bounds" ); 
     
     m_mem[ii] = eT(arma_rng::randu<eT>());
     }
@@ -496,7 +534,7 @@ subview_elem1<eT,T1>::randn()
   arma_debug_check
     (
     ( (aa.is_vec() == false) && (aa.is_empty() == false) ),
-    "Mat::elem(): given object is not a vector"
+    "Mat::elem(): given object must be a vector"
     );
   
   const uword* aa_mem    = aa.memptr();
@@ -508,7 +546,7 @@ subview_elem1<eT,T1>::randn()
     const uword ii = aa_mem[iq];
     const uword jj = aa_mem[jq];
     
-    arma_debug_check( ( (ii >= m_n_elem) || (jj >= m_n_elem) ), "Mat::elem(): index out of bounds" );
+    arma_debug_check_bounds( ( (ii >= m_n_elem) || (jj >= m_n_elem) ), "Mat::elem(): index out of bounds" );
     
     arma_rng::randn<eT>::dual_val( m_mem[ii], m_mem[jj] );
     }
@@ -517,7 +555,7 @@ subview_elem1<eT,T1>::randn()
     {
     const uword ii = aa_mem[iq];
     
-    arma_debug_check( (ii >= m_n_elem) , "Mat::elem(): index out of bounds" ); 
+    arma_debug_check_bounds( (ii >= m_n_elem) , "Mat::elem(): index out of bounds" ); 
     
     m_mem[ii] = eT(arma_rng::randn<eT>());
     }
@@ -753,7 +791,7 @@ subview_elem1<eT,T1>::extract(Mat<eT>& actual_out, const subview_elem1<eT,T1>& i
   arma_debug_check
     (
     ( (aa.is_vec() == false) && (aa.is_empty() == false) ),
-    "Mat::elem(): given object is not a vector"
+    "Mat::elem(): given object must be a vector"
     );
   
   const uword* aa_mem    = aa.memptr();
@@ -781,7 +819,7 @@ subview_elem1<eT,T1>::extract(Mat<eT>& actual_out, const subview_elem1<eT,T1>& i
     const uword ii = aa_mem[i];
     const uword jj = aa_mem[j];
     
-    arma_debug_check( ( (ii >= m_n_elem) || (jj >= m_n_elem) ), "Mat::elem(): index out of bounds" );
+    arma_debug_check_bounds( ( (ii >= m_n_elem) || (jj >= m_n_elem) ), "Mat::elem(): index out of bounds" );
     
     out_mem[i] = m_mem[ii];
     out_mem[j] = m_mem[jj];
@@ -791,12 +829,12 @@ subview_elem1<eT,T1>::extract(Mat<eT>& actual_out, const subview_elem1<eT,T1>& i
     {
     const uword ii = aa_mem[i];
     
-    arma_debug_check( (ii >= m_n_elem) , "Mat::elem(): index out of bounds" );
+    arma_debug_check_bounds( (ii >= m_n_elem) , "Mat::elem(): index out of bounds" );
     
     out_mem[i] = m_mem[ii];
     }
   
-  if(alias == true)
+  if(alias)
     {
     actual_out.steal_mem(out);
     delete tmp_out;
@@ -819,7 +857,7 @@ subview_elem1<eT,T1>::mat_inplace_op(Mat<eT>& out, const subview_elem1& in)
   arma_debug_check
     (
     ( (aa.is_vec() == false) && (aa.is_empty() == false) ),
-    "Mat::elem(): given object is not a vector"
+    "Mat::elem(): given object must be a vector"
     );
   
   const uword* aa_mem    = aa.memptr();
@@ -841,7 +879,7 @@ subview_elem1<eT,T1>::mat_inplace_op(Mat<eT>& out, const subview_elem1& in)
     const uword ii = aa_mem[i];
     const uword jj = aa_mem[j];
     
-    arma_debug_check( ( (ii >= m_n_elem) || (jj >= m_n_elem) ), "Mat::elem(): index out of bounds" );
+    arma_debug_check_bounds( ( (ii >= m_n_elem) || (jj >= m_n_elem) ), "Mat::elem(): index out of bounds" );
     
     if(is_same_type<op_type, op_internal_plus >::yes) { out_mem[i] += m_mem[ii]; out_mem[j] += m_mem[jj]; }
     if(is_same_type<op_type, op_internal_minus>::yes) { out_mem[i] -= m_mem[ii]; out_mem[j] -= m_mem[jj]; }
@@ -853,7 +891,7 @@ subview_elem1<eT,T1>::mat_inplace_op(Mat<eT>& out, const subview_elem1& in)
     {
     const uword ii = aa_mem[i];
     
-    arma_debug_check( (ii >= m_n_elem) , "Mat::elem(): index out of bounds" );
+    arma_debug_check_bounds( (ii >= m_n_elem) , "Mat::elem(): index out of bounds" );
     
     if(is_same_type<op_type, op_internal_plus >::yes) { out_mem[i] += m_mem[ii]; }
     if(is_same_type<op_type, op_internal_minus>::yes) { out_mem[i] -= m_mem[ii]; }
