@@ -1,3 +1,5 @@
+// SPDX-License-Identifier: Apache-2.0
+// 
 // Copyright 2011-2017 Ryan Curtin (http://www.ratml.org/)
 // Copyright 2017 National ICT Australia (NICTA)
 //
@@ -22,7 +24,7 @@ using namespace arma;
 // Does the matrix correctly report when it is empty?
 TEST_CASE("empty_test")
   {
-  bool testPassed = true;
+  // bool testPassed = true;
 
   sp_imat test;
   REQUIRE( test.is_empty() );
@@ -209,7 +211,7 @@ TEST_CASE("division_test")
 
   REQUIRE( std::isnan((double) a(0, 0)) );
   REQUIRE( (double) a(0, 1) == Approx(0.5) );
-  REQUIRE( (double) a(1, 0) == Approx(1e-5) );
+  REQUIRE( (double) a(1, 0) == Approx(0.0).margin(0.001) );
   REQUIRE( std::isnan((double) a(1, 1)) );
   }
 
@@ -953,7 +955,7 @@ TEST_CASE("sp_mat_eye_test")
       if (i == j)
         REQUIRE( (double) e(i, j) == Approx(1.0) );
       else
-        REQUIRE( (double) e(i, j) == Approx(1e-5) );
+        REQUIRE( (double) e(i, j) == Approx(0.0).margin(0.001) );
       }
     }
 
@@ -1217,7 +1219,7 @@ TEST_CASE("min_test")
   a(2) = 1.0;
 
   double res = min(a);
-  REQUIRE( res == Approx(1e-5) );
+  REQUIRE( res == Approx(0.0).margin(0.001) );
 
   a(0) = -3.0;
   a(2) = -1.0;
@@ -1240,7 +1242,7 @@ TEST_CASE("min_test")
   b(2) = 1.0;
 
   res = min(b);
-  REQUIRE( res == Approx(1e-5) );
+  REQUIRE( res == Approx(0.0).margin(0.001) );
 
   b(0) = -3.0;
   b(2) = -1.0;
@@ -1275,18 +1277,18 @@ TEST_CASE("min_test")
   REQUIRE( r.n_rows == 1 );
   REQUIRE( r.n_cols == 5 );
   REQUIRE( (double) r(0, 0) == Approx(0.6) );
-  REQUIRE( (double) r(0, 1) == Approx(1e-5) );
+  REQUIRE( (double) r(0, 1) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) r(0, 2) == Approx(-3.4) );
   REQUIRE( (double) r(0, 3) == Approx(-4.1) );
-  REQUIRE( (double) r(0, 4) == Approx(1e-5) );
+  REQUIRE( (double) r(0, 4) == Approx(0.0).margin(0.001) );
 
   r = min(c, 1);
   REQUIRE( r.n_rows == 6 );
   REQUIRE( r.n_cols == 1 );
-  REQUIRE( (double) r(0, 0) == Approx(1e-5) );
-  REQUIRE( (double) r(1, 0) == Approx(1e-5) );
+  REQUIRE( (double) r(0, 0) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) r(1, 0) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) r(2, 0) == Approx(-4.0) );
-  REQUIRE( (double) r(3, 0) == Approx(1e-5) );
+  REQUIRE( (double) r(3, 0) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) r(4, 0) == Approx(-1.4) );
   REQUIRE( (double) r(5, 0) == Approx(-4.1) );
   }
@@ -1300,7 +1302,7 @@ TEST_CASE("max_test")
   a(2) = -1.0;
 
   double resa = max(a);
-  REQUIRE( resa == Approx(1e-5) );
+  REQUIRE( resa == Approx(0.0).margin(0.001) );
 
   a(0) = 3.0;
   a(2) = 1.0;
@@ -1323,7 +1325,7 @@ TEST_CASE("max_test")
   b(2) = -1.0;
 
   resa = max(b);
-  REQUIRE( resa == Approx(1e-5) );
+  REQUIRE( resa == Approx(0.0).margin(0.001) );
 
   b(0) = 3.0;
   b(2) = 1.0;
@@ -1358,10 +1360,10 @@ TEST_CASE("max_test")
   REQUIRE( res.n_rows == 1 );
   REQUIRE( res.n_cols == 5 );
   REQUIRE( (double) res(0, 0) == Approx(4.0) );
-  REQUIRE( (double) res(0, 1) == Approx(1e-5) );
+  REQUIRE( (double) res(0, 1) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) res(0, 2) == Approx(1.3) );
-  REQUIRE( (double) res(0, 3) == Approx(1e-5) );
-  REQUIRE( (double) res(0, 4) == Approx(1e-5) );
+  REQUIRE( (double) res(0, 3) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) res(0, 4) == Approx(0.0).margin(0.001) );
 
   res = max(c, 1);
   REQUIRE( res.n_rows == 6 );
@@ -1370,7 +1372,7 @@ TEST_CASE("max_test")
   REQUIRE( (double) res(1, 0) == Approx(3.0) );
   REQUIRE( (double) res(2, 0) == Approx(4.0) );
   REQUIRE( (double) res(3, 0) == Approx(1.3) );
-  REQUIRE( (double) res(4, 0) == Approx(1e-5) );
+  REQUIRE( (double) res(4, 0) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) res(5, 0) == Approx(1.2) );
   }
 
@@ -1383,15 +1385,15 @@ TEST_CASE("spmat_min_cx_test")
   a(2) = cx_double(1.0, 1.0);
 
   cx_double res = min(a);
-  REQUIRE( res.real() == Approx(1e-5) );
-  REQUIRE( res.imag() == Approx(1e-5) );
+  REQUIRE( res.real() == Approx(0.0).margin(0.001) );
+  REQUIRE( res.imag() == Approx(0.0).margin(0.001) );
 
   a(0) = cx_double(-3.0, -2.0);
   a(2) = cx_double(-1.0, -1.0);
 
   res = min(a);
-  REQUIRE( res.real() == Approx(1e-5) );
-  REQUIRE( res.imag() == Approx(1e-5) );
+  REQUIRE( res.real() == Approx(0.0).margin(0.001) );
+  REQUIRE( res.imag() == Approx(0.0).margin(0.001) );
 
   a(0) = cx_double(1.0, 0.5);
   a(1) = cx_double(2.4, 1.4);
@@ -1409,15 +1411,15 @@ TEST_CASE("spmat_min_cx_test")
   b(2) = cx_double(1.0, 1.0);
 
   res = min(b);
-  REQUIRE( res.real() == Approx(1e-5) );
-  REQUIRE( res.imag() == Approx(1e-5) );
+  REQUIRE( res.real() == Approx(0.0).margin(0.001) );
+  REQUIRE( res.imag() == Approx(0.0).margin(0.001) );
 
   b(0) = cx_double(-3.0, -2.0);
   b(2) = cx_double(-1.0, -1.0);
 
   res = min(b);
-  REQUIRE( res.real() == Approx(1e-5) );
-  REQUIRE( res.imag() == Approx(1e-5) );
+  REQUIRE( res.real() == Approx(0.0).margin(0.001) );
+  REQUIRE( res.imag() == Approx(0.0).margin(0.001) );
 
   b(0) = cx_double(1.0, 0.5);
   b(1) = cx_double(2.4, 1.4);
@@ -1441,24 +1443,24 @@ TEST_CASE("spmat_min_cx_test")
   SpMat<cx_double> r = min(c, 0);
   REQUIRE( r.n_rows == 1 );
   REQUIRE( r.n_cols == 3 );
-  REQUIRE( ((cx_double) r(0, 0)).real() == Approx(1e-5) );
-  REQUIRE( ((cx_double) r(0, 0)).imag() == Approx(1e-5) );
+  REQUIRE( ((cx_double) r(0, 0)).real() == Approx(0.0).margin(0.001) );
+  REQUIRE( ((cx_double) r(0, 0)).imag() == Approx(0.0).margin(0.001) );
   REQUIRE( ((cx_double) r(0, 1)).real() == Approx(0.25) );
   REQUIRE( ((cx_double) r(0, 1)).imag() == Approx(0.25) );
-  REQUIRE( ((cx_double) r(0, 2)).real() == Approx(1e-5) );
-  REQUIRE( ((cx_double) r(0, 2)).imag() == Approx(1e-5) );
+  REQUIRE( ((cx_double) r(0, 2)).real() == Approx(0.0).margin(0.001) );
+  REQUIRE( ((cx_double) r(0, 2)).imag() == Approx(0.0).margin(0.001) );
 
   r = min(c, 1);
   REQUIRE( r.n_rows == 4 );
   REQUIRE( r.n_cols == 1 );
   REQUIRE( ((cx_double) r(0, 0)).real() == Approx(0.5) );
   REQUIRE( ((cx_double) r(0, 0)).imag() == Approx(0.5) );
-  REQUIRE( ((cx_double) r(1, 0)).real() == Approx(1e-5) );
-  REQUIRE( ((cx_double) r(1, 0)).imag() == Approx(1e-5) );
-  REQUIRE( ((cx_double) r(2, 0)).real() == Approx(1e-5) );
-  REQUIRE( ((cx_double) r(2, 0)).imag() == Approx(1e-5) );
-  REQUIRE( ((cx_double) r(3, 0)).real() == Approx(1e-5) );
-  REQUIRE( ((cx_double) r(3, 0)).imag() == Approx(1e-5) );
+  REQUIRE( ((cx_double) r(1, 0)).real() == Approx(0.0).margin(0.001) );
+  REQUIRE( ((cx_double) r(1, 0)).imag() == Approx(0.0).margin(0.001) );
+  REQUIRE( ((cx_double) r(2, 0)).real() == Approx(0.0).margin(0.001) );
+  REQUIRE( ((cx_double) r(2, 0)).imag() == Approx(0.0).margin(0.001) );
+  REQUIRE( ((cx_double) r(3, 0)).real() == Approx(0.0).margin(0.001) );
+  REQUIRE( ((cx_double) r(3, 0)).imag() == Approx(0.0).margin(0.001) );
 }
 
 
@@ -1478,8 +1480,8 @@ TEST_CASE("spmat_max_cx_test")
   a(2) = cx_double(0);
 
   res = max(a);
-  REQUIRE( res.real() == Approx(1e-5) );
-  REQUIRE( res.imag() == Approx(1e-5) );
+  REQUIRE( res.real() == Approx(0.0).margin(0.001) );
+  REQUIRE( res.imag() == Approx(0.0).margin(0.001) );
 
   a(0) = cx_double(1.0, 0.5);
   a(1) = cx_double(2.4, 1.4);
@@ -1504,8 +1506,8 @@ TEST_CASE("spmat_max_cx_test")
   b(2) = cx_double(0);
 
   res = max(b);
-  REQUIRE( res.real() == Approx(1e-5) );
-  REQUIRE( res.imag() == Approx(1e-5) );
+  REQUIRE( res.real() == Approx(0.0).margin(0.001) );
+  REQUIRE( res.imag() == Approx(0.0).margin(0.001) );
 
   b(0) = cx_double(1.0, 0.5);
   b(1) = cx_double(2.4, 1.4);
@@ -1532,8 +1534,8 @@ TEST_CASE("spmat_max_cx_test")
   REQUIRE( ((cx_double) r(0, 0)).imag() == Approx(2.0) );
   REQUIRE( ((cx_double) r(0, 1)).real() == Approx(-3.0) );
   REQUIRE( ((cx_double) r(0, 1)).imag() == Approx(-3.0) );
-  REQUIRE( ((cx_double) r(0, 2)).real() == Approx(1e-5) );
-  REQUIRE( ((cx_double) r(0, 2)).imag() == Approx(1e-5) );
+  REQUIRE( ((cx_double) r(0, 2)).real() == Approx(0.0).margin(0.001) );
+  REQUIRE( ((cx_double) r(0, 2)).imag() == Approx(0.0).margin(0.001) );
 
   r = max(c, 1);
   REQUIRE( r.n_rows == 4 );
@@ -1690,16 +1692,16 @@ TEST_CASE("spmat_unary_val_operators_test")
 
   REQUIRE( b.n_nonzero == 2 );
   REQUIRE( (double) b(0, 0) == Approx(6.0) );
-  REQUIRE( (double) b(0, 1) == Approx(1e-5) );
-  REQUIRE( (double) b(1, 0) == Approx(1e-5) );
+  REQUIRE( (double) b(0, 1) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) b(1, 0) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) b(1, 1) == Approx(-9.0) );
 
   b = a / 3.0;
 
   REQUIRE( b.n_nonzero == 2 );
   REQUIRE( (double) b(0, 0) == Approx(2.0 / 3.0) );
-  REQUIRE( (double) b(0, 1) == Approx(1e-5) );
-  REQUIRE( (double) b(1, 0) == Approx(1e-5) );
+  REQUIRE( (double) b(0, 1) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) b(1, 0) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) b(1, 1) == Approx(-1.0) );
   }
 
@@ -1792,7 +1794,7 @@ TEST_CASE("spmat_unary_operator_test_2")
   REQUIRE( c(2, 1) == Approx(3.0) );
   REQUIRE( c(0, 2) == Approx(6.5) );
   REQUIRE( c(1, 2) == Approx(7.0) );
-  REQUIRE( c(2, 2) == Approx(1e-5) );
+  REQUIRE( c(2, 2) == Approx(0.0).margin(0.001) );
 
   c = a - b;
 
@@ -1810,11 +1812,11 @@ TEST_CASE("spmat_unary_operator_test_2")
 
   REQUIRE( d.n_nonzero == 4 );
   REQUIRE( (double) d(0, 0) == Approx(3.0) );
-  REQUIRE( (double) d(1, 0) == Approx(1e-5) );
-  REQUIRE( (double) d(2, 0) == Approx(1e-5) );
-  REQUIRE( (double) d(0, 1) == Approx(1e-5) );
-  REQUIRE( (double) d(1, 1) == Approx(1e-5) );
-  REQUIRE( (double) d(2, 1) == Approx(1e-5) );
+  REQUIRE( (double) d(1, 0) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) d(2, 0) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) d(0, 1) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) d(1, 1) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) d(2, 1) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) d(0, 2) == Approx(10.5) );
   REQUIRE( (double) d(1, 2) == Approx(12.0) );
   REQUIRE( (double) d(2, 2) == Approx(-9.0) );
@@ -1823,11 +1825,11 @@ TEST_CASE("spmat_unary_operator_test_2")
 
   REQUIRE( d.n_nonzero == 4 );
   REQUIRE( (double) d(0, 0) == Approx((1.0 / 3.0)) );
-  REQUIRE( (double) d(1, 0) == Approx(1e-5) );
-  REQUIRE( (double) d(2, 0) == Approx(1e-5) );
-  REQUIRE( (double) d(0, 1) == Approx(1e-5) );
-  REQUIRE( (double) d(1, 1) == Approx(1e-5) );
-  REQUIRE( (double) d(2, 1) == Approx(1e-5) );
+  REQUIRE( (double) d(1, 0) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) d(2, 0) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) d(0, 1) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) d(1, 1) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) d(2, 1) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) d(0, 2) == Approx((3.5 / 3.0)) );
   REQUIRE( (double) d(1, 2) == Approx((4.0 / 3.0)) );
   REQUIRE( (double) d(2, 2) == Approx(-1.0) );
@@ -1849,9 +1851,9 @@ TEST_CASE("spmat_unary_operator_test_2")
   REQUIRE( (double) c(0, 0) == Approx(3.0) );
   REQUIRE( (double) c(1, 0) == Approx(3.0) );
   REQUIRE( (double) c(2, 0) == Approx(3.0) );
-  REQUIRE( (double) c(0, 1) == Approx(1e-5) );
-  REQUIRE( (double) c(1, 1) == Approx(1e-5) );
-  REQUIRE( (double) c(2, 1) == Approx(1e-5) );
+  REQUIRE( (double) c(0, 1) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) c(1, 1) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) c(2, 1) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) c(0, 2) == Approx(13.5) );
   REQUIRE( (double) c(1, 2) == Approx(13.5) );
   REQUIRE( (double) c(2, 2) == Approx(13.5) );
@@ -1879,7 +1881,7 @@ TEST_CASE("spmat_mat_operator_tests")
   REQUIRE( (double) c(2, 0) == Approx(2.0) );
   REQUIRE( (double) c(0, 1) == Approx(2.0) );
   REQUIRE( (double) c(1, 1) == Approx(2.0) );
-  REQUIRE( (double) c(2, 1) == Approx(1e-5) );
+  REQUIRE( (double) c(2, 1) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) c(0, 2) == Approx(2.0) );
   REQUIRE( (double) c(1, 2) == Approx(5.5) );
   REQUIRE( (double) c(2, 2) == Approx(6.5) );
@@ -1891,7 +1893,7 @@ TEST_CASE("spmat_mat_operator_tests")
   REQUIRE( (double) c(2, 0) == Approx(2.0) );
   REQUIRE( (double) c(0, 1) == Approx(2.0) );
   REQUIRE( (double) c(1, 1) == Approx(2.0) );
-  REQUIRE( (double) c(2, 1) == Approx(1e-5) );
+  REQUIRE( (double) c(2, 1) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) c(0, 2) == Approx(2.0) );
   REQUIRE( (double) c(1, 2) == Approx(5.5) );
   REQUIRE( (double) c(2, 2) == Approx(6.5) );
@@ -1899,7 +1901,7 @@ TEST_CASE("spmat_mat_operator_tests")
   c = b;
   c -= a;
 
-  REQUIRE( (double) c(0, 0) == Approx(1e-5) );
+  REQUIRE( (double) c(0, 0) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) c(1, 0) == Approx(2.0) );
   REQUIRE( (double) c(2, 0) == Approx(2.0) );
   REQUIRE( (double) c(0, 1) == Approx(2.0) );
@@ -1911,7 +1913,7 @@ TEST_CASE("spmat_mat_operator_tests")
 
   c = b - a;
 
-  REQUIRE( (double) c(0, 0) == Approx(1e-5) );
+  REQUIRE( (double) c(0, 0) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) c(1, 0) == Approx(2.0) );
   REQUIRE( (double) c(2, 0) == Approx(2.0) );
   REQUIRE( (double) c(0, 1) == Approx(2.0) );
@@ -1950,12 +1952,12 @@ TEST_CASE("spmat_mat_operator_tests")
   c %= a;
 
   REQUIRE( (double) c(0, 0) == Approx(4.0) );
-  REQUIRE( (double) c(1, 0) == Approx(1e-5) );
-  REQUIRE( (double) c(2, 0) == Approx(1e-5) );
-  REQUIRE( (double) c(0, 1) == Approx(1e-5) );
-  REQUIRE( (double) c(1, 1) == Approx(1e-5) );
+  REQUIRE( (double) c(1, 0) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) c(2, 0) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) c(0, 1) == Approx(0.0).margin(0.001) );
+  REQUIRE( (double) c(1, 1) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) c(2, 1) == Approx(-4.0) );
-  REQUIRE( (double) c(0, 2) == Approx(1e-5) );
+  REQUIRE( (double) c(0, 2) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) c(1, 2) == Approx(7.0) );
   REQUIRE( (double) c(2, 2) == Approx(9.0) );
 
@@ -2010,7 +2012,7 @@ TEST_CASE("spmat_sparse_dense_in_place")
       if ((double) a(r, c) != 0)
         REQUIRE( (double) a(r, c) == Approx(d(r, c)) );
       else
-        REQUIRE( d(r, c) == Approx(1e-5) );
+        REQUIRE( d(r, c) == Approx(0.0).margin(0.001) );
       }
     }
 
@@ -2030,7 +2032,7 @@ TEST_CASE("spmat_sparse_dense_in_place")
       if ((double) a(r, c) != 0)
         REQUIRE( (double) a(r, c) == Approx(d(r, c)) );
       else
-        REQUIRE( d(r, c) == Approx(1e-5) );
+        REQUIRE( d(r, c) == Approx(0.0).margin(0.001) );
       }
     }
 
@@ -2047,7 +2049,7 @@ TEST_CASE("spmat_sparse_dense_in_place")
       if ((double) a(r, c) != 0)
         REQUIRE( (double) a(r, c) == Approx(d(r, c)) );
       else
-        REQUIRE( d(r, c) == Approx(1e-5) );
+        REQUIRE( d(r, c) == Approx(0.0).margin(0.001) );
       }
     }
 
@@ -2064,7 +2066,7 @@ TEST_CASE("spmat_sparse_dense_in_place")
       if ((double) a(r, c) != 0)
         REQUIRE( (double) a(r, c) == Approx(d(r, c)) );
       else
-        REQUIRE(d(r, c) == Approx(1e-5) );
+        REQUIRE(d(r, c) == Approx(0.0).margin(0.001) );
       }
     }
   }
@@ -2093,7 +2095,7 @@ TEST_CASE("spmat_sparse_dense_not_in_place")
       if ((double) y(r, c) != 0)
         REQUIRE( (double) y(r, c) == Approx(z(r, c)) );
       else
-        REQUIRE( z(r, c) == Approx(1e-5) );
+        REQUIRE( z(r, c) == Approx(0.0).margin(0.001) );
       }
     }
 
@@ -2107,7 +2109,7 @@ TEST_CASE("spmat_sparse_dense_not_in_place")
       if ((double) y(r, c) != 0)
         REQUIRE( (double) y(r, c) == Approx(z(r, c)) );
       else
-        REQUIRE( z(r, c) == Approx(1e-5) );
+        REQUIRE( z(r, c) == Approx(0.0).margin(0.001) );
       }
     }
 
@@ -2121,7 +2123,7 @@ TEST_CASE("spmat_sparse_dense_not_in_place")
       if ((double) y(r, c) != 0)
         REQUIRE( (double) y(r, c) == Approx(z(r, c)) );
       else
-        REQUIRE( z(r, c) == Approx(1e-5) );
+        REQUIRE( z(r, c) == Approx(0.0).margin(0.001) );
       }
     }
 
@@ -2135,7 +2137,7 @@ TEST_CASE("spmat_sparse_dense_not_in_place")
       if ((double) y(r, c) != 0)
         REQUIRE( (double) y(r, c) == Approx(z(r, c)) );
       else
-        REQUIRE( z(r, c) == Approx(1e-5) );
+        REQUIRE( z(r, c) == Approx(0.0).margin(0.001) );
       }
     }
 
@@ -2149,7 +2151,7 @@ TEST_CASE("spmat_sparse_dense_not_in_place")
       if ((double) y(r, c) != 0)
         REQUIRE( (double) y(r, c) == Approx(z(r, c)) );
       else
-        REQUIRE( z(r, c) == Approx(1e-5) );
+        REQUIRE( z(r, c) == Approx(0.0).margin(0.001) );
       }
     }
 
@@ -2163,7 +2165,7 @@ TEST_CASE("spmat_sparse_dense_not_in_place")
       if ((double) y(r, c) != 0)
         REQUIRE( (double) y(r, c) == Approx(z(r, c)) );
       else
-        REQUIRE( z(r, c) == Approx(1e-5) );
+        REQUIRE( z(r, c) == Approx(0.0).margin(0.001) );
     }
   }
 
@@ -2177,7 +2179,7 @@ TEST_CASE("spmat_sparse_dense_not_in_place")
       if ((double) y(r, c) != 0)
         REQUIRE( (double) y(r, c) == Approx(z(r, c)) );
       else
-        REQUIRE( z(r, c) == Approx(1e-5) );
+        REQUIRE( z(r, c) == Approx(0.0).margin(0.001) );
       }
     }
 
@@ -2191,7 +2193,7 @@ TEST_CASE("spmat_sparse_dense_not_in_place")
       if ((double) y(r, c) != 0)
         REQUIRE( (double) y(r, c) == Approx(z(r, c)) );
       else
-        REQUIRE( z(r, c) == Approx(1e-5) );
+        REQUIRE( z(r, c) == Approx(0.0).margin(0.001) );
       }
     }
 
@@ -2205,7 +2207,7 @@ TEST_CASE("spmat_sparse_dense_not_in_place")
       if ((double) y(r, c) != 0)
         REQUIRE( (double) y(r, c) == Approx(z(r, c)) );
       else
-        REQUIRE( z(r, c) == Approx(1e-5) );
+        REQUIRE( z(r, c) == Approx(0.0).margin(0.001) );
       }
     }
   }
@@ -2492,7 +2494,7 @@ TEST_CASE("spmat_batch_insert_zeroes_test")
   REQUIRE( (double) m(2, 1) == Approx(1.5) );
   REQUIRE( (double) m(7, 1) == Approx(-15.15) );
   REQUIRE( (double) m(0, 4) == Approx(2.2) );
-  REQUIRE( (double) m(9, 4) == Approx(1e-5) );
+  REQUIRE( (double) m(9, 4) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) m(0, 5) == Approx(5.0) );
 
   // Auto size detection.
@@ -2504,7 +2506,7 @@ TEST_CASE("spmat_batch_insert_zeroes_test")
   REQUIRE( (double) n(2, 1) == Approx(1.5) );
   REQUIRE( (double) n(7, 1) == Approx(-15.15) );
   REQUIRE( (double) n(0, 4) == Approx(2.2) );
-  REQUIRE( (double) n(9, 4) == Approx(1e-5) );
+  REQUIRE( (double) n(9, 4) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) n(0, 5) == Approx(5.0) );
   }
 
@@ -2539,7 +2541,7 @@ TEST_CASE("spmat_batch_insert_unsorted_case_zeroes")
   REQUIRE( (double) m(2, 1) == Approx(1.5) );
   REQUIRE( (double) m(7, 1) == Approx(-15.15) );
   REQUIRE( (double) m(0, 4) == Approx(2.2) );
-  REQUIRE( (double) m(9, 4) == Approx(1e-5) );
+  REQUIRE( (double) m(9, 4) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) m(0, 5) == Approx(5.0) );
   REQUIRE( m.col_ptrs[11] == std::numeric_limits<uword>::max() );
 
@@ -2552,7 +2554,7 @@ TEST_CASE("spmat_batch_insert_unsorted_case_zeroes")
   REQUIRE( (double) n(2, 1) == Approx(1.5) );
   REQUIRE( (double) n(7, 1) == Approx(-15.15) );
   REQUIRE( (double) n(0, 4) == Approx(2.2) );
-  REQUIRE( (double) n(9, 4) == Approx(1e-5) );
+  REQUIRE( (double) n(9, 4) == Approx(0.0).margin(0.001) );
   REQUIRE( (double) n(0, 5) == Approx(5.0) );
   REQUIRE( n.col_ptrs[7] == std::numeric_limits<uword>::max() );
   }
